package net.chrisrichardson.foodToGo.order;



import java.io.*;
import java.util.*;
import net.chrisrichardson.foodToGo.creditCardProcessing.*;
import net.chrisrichardson.foodToGo.order.*;
import net.chrisrichardson.foodToGo.util.*;


public class PendingOrder  {
    
    public static final int OK = 0;
    public static final int CONFIRM_CHANGE = 1;
    public static final int INVALID_DELIVERY_ADDRESS_OR_TIME = 2;

    public static final int NEW = 0;
    public static final int DELIVERY_ADDRESS_AND_TIME_SPECIFIED = 1;
    public static final int RESTAURANT_SELECTED = 2;
    public static final int READY_FOR_CHECKOUT = 3;
    public static final int NEEDS_PAYMENT = 4;
    public static final int READY_TO_PLACE = 5;
    public static final int PLACED = 6;

    
	private Date deliveryDate;

	private Restaurant restaurant;
	private ArrayList lineItems = new ArrayList();

	private int state = NEW;

	private String deliveryStreet1;
	private String deliveryStreet2;
	private String deliveryCity;
	private String deliveryState;
	private String deliveryZip;

    private String paymentType;
    private String paymentName;
    private String paymentNumber;
    private int paymentMonth;
    private int paymentYear;
    private String paymentStreet1;
    private String paymentStreet2;
    private String paymentCity;
    private String paymentState;
    private String paymentZip;


	public PendingOrder() {
	}



	/**
	 * Make sure the delivery time is sufficiently 
	 * far in the future
	 * 
	 * @param deliveryTime
	 * @return 
	 */
	Date verifyDeliveryTime(Date deliveryTime) {
		Date now = new Date();
		Date earliest = new Date(now.getTime() + (3600 * 1000));
		if (deliveryTime.before(earliest))
			return earliest;
		else
			return deliveryTime;
	}

	Address verifyDeliveryAddress(Address deliveryAddress) {
		return deliveryAddress;
	}

	/**
	 * Returns true if the delivery address and time
	 * can be served by the selected restaurant
	 * 
	 * @param deliveryAddress
	 * @param deliveryTime
	 * @return 
	 */
	protected boolean isCompatibleWithSelectedRestaurant(
		Address deliveryAddress,
		Date deliveryTime) {
		return getRestaurant() == null
			|| getRestaurant().isInServiceArea(deliveryAddress, deliveryTime);
	}

	/**
	 * Change the delivery info
	 * 
	 * @param deliveryAddress
	 * @param deliveryTime
	 * @param force
	 * @return 
	 */
	public int updateDeliveryAddressAndDate(
		Address deliveryAddress,
		Date deliveryTime,
		boolean force) {

		Date verifiedDeliveryTime = verifyDeliveryTime(deliveryTime);
		Address verifiedDeliveryAddress = verifyDeliveryAddress(deliveryAddress);

		if (!RestaurantRepository
			.isRestaurantAvailable(verifiedDeliveryAddress, verifiedDeliveryTime))
			return PendingOrder.INVALID_DELIVERY_ADDRESS_OR_TIME;

		if (getRestaurant() != null) {
			if (getRestaurant()
				.isInServiceArea(verifiedDeliveryAddress, verifiedDeliveryTime)) {
				setDeliveryAddress(verifiedDeliveryAddress);
				setDeliveryDate(verifiedDeliveryTime);
				// getState() unchanged
				return PendingOrder.OK;
			} else if (!force) {
				return PendingOrder.CONFIRM_CHANGE;
			}
		}

		setDeliveryAddress(verifiedDeliveryAddress);
		setDeliveryDate(verifiedDeliveryTime);
		setRestaurant(null);
		setState(PendingOrder.DELIVERY_ADDRESS_AND_TIME_SPECIFIED);
		return PendingOrder.OK;
	}

	/**
	 * Change the restaurant
	 * 
	 * @param restaurant
	 * @return 
	 * @exception InvalidPendingOrderStateException
	 */
	public boolean updateRestaurant(Restaurant restaurant)
		throws InvalidPendingOrderStateException {

		if (getState() == PendingOrder.NEW || getState() == PendingOrder.PLACED)
			throw new InvalidPendingOrderStateException("State = " + getState());

		if (restaurant.isInServiceArea(getDeliveryAddress(), getDeliveryDate())) {
			setRestaurant(restaurant);
			MenuItemCollection menuItems = restaurant.findMenuItems();
			setMenuItems(menuItems);
			setState(PendingOrder.RESTAURANT_SELECTED);
			return true;
		} else {
			// restaurant can't deliver to the specified address/time
			return false;
		}
	}

	/**
	 * Create pending order line items from the menu items
	 * 
	 * @param menuItems
	 */
	protected void setMenuItems(MenuItemCollection menuItems) {
		Iterator it = menuItems.iterator();
		PendingOrderLineItemCollection newLineItems =
			new PendingOrderLineItemCollection();
		int index = 0;
		while (it.hasNext()) {
			MenuItem miv = (MenuItem) it.next();
			newLineItems.add(makeLineItem(index++, miv));
		}
		setLineItems(newLineItems);
	}

	//    protected PendingOrderLineItem makeLineItem(MenuItem miv) {
	//        return DomainManager.getInstance().makePendingOrderLineItem(miv);
	//    }


	/**
	 * Change the quantities
	 * 
	 * @param quantities
	 * @exception InvalidPendingOrderStateException
	 */
	public void updateQuantities(int[] quantities)
		throws InvalidPendingOrderStateException {
		if (getState() != PendingOrder.RESTAURANT_SELECTED
			&& getState() != PendingOrder.READY_FOR_CHECKOUT
			&& getState() != PendingOrder.READY_TO_PLACE)
			throw new InvalidPendingOrderStateException();

		PendingOrderLineItemCollection lineItems = getLineItems();

		if (lineItems.size() != quantities.length)
			throw new InvalidPendingOrderStateException(
				"Not the same size. Wanted: " +  lineItems.size()
					+ ", but got "
					+ quantities.length);

        for (int i = 0; i < quantities.length; i++) {
			lineItems.get(i).setQuantity(quantities[i]);
		}
        
		setState(
			meetsMinimumOrder()
				? PendingOrder.READY_FOR_CHECKOUT
				: PendingOrder.RESTAURANT_SELECTED);

		// all quantities >= 0
		// action:
		// recalculate totals
		// warning if less than minimum

	}

	public boolean checkout() throws InvalidPendingOrderStateException {
		if (getState() != PendingOrder.READY_FOR_CHECKOUT
			&& getState() != PendingOrder.RESTAURANT_SELECTED)
			throw new InvalidPendingOrderStateException();

		return getState() == PendingOrder.READY_FOR_CHECKOUT;
	}

	protected boolean meetsMinimumOrder() {
		return getSubtotal() > 0;
	}

	public double getSubtotal() {
		if (getLineItems() == null)
			return 0;

		double subTotal = 0;
		Iterator it = getLineItems().iterator();
		while (it.hasNext()) {
			PendingOrderLineItem li = (PendingOrderLineItem) it.next();
			subTotal += li.getExtendedPrice();
		}
		return subTotal;
	}

	public double getDeliveryCharges() {
		return 5.00;
	}

	public double getSalesTax() {
		return getSubtotal() * 0.0825;
	}

	public double getTotal() {
		return getSubtotal() + getDeliveryCharges() + getSalesTax();
	}

	/**
	 * Change the payment information
	 * 
	 * @param paymentInformation
	 * @exception InvalidPendingOrderStateException
	 */
	public void updatePaymentInformation(PaymentInformation paymentInformation)
		throws InvalidPendingOrderStateException {
		if (getState() != PendingOrder.READY_FOR_CHECKOUT)
			throw new InvalidPendingOrderStateException();
		setPaymentInformation(paymentInformation);
		setState(PendingOrder.READY_TO_PLACE);
	}

	/**
	 * Authorize the credit card and place the order if successful
	 * 
	 * @return 
	 * @exception InvalidPendingOrderStateException
	 */
	public String placeOrder() throws InvalidPendingOrderStateException {
		if (getState() != PendingOrder.READY_TO_PLACE)
			throw new InvalidPendingOrderStateException();
		try {
			AuthorizationTransaction txn =
				CreditCardProcessor.authorize(getPaymentInformation());
			setAuthorizationTransaction(txn);
			setState(PendingOrder.PLACED);
			return OrderFactory.createOrder(this);
		} catch (CreditCardProcessingException e) {
			// What should we do
			throw new InvalidPendingOrderStateException();
		}
	}


	public int getState() {
		return state;
	}

	public void setState(int state) {
		this.state = state;
	}


	public Date getDeliveryDate() {
		return deliveryDate;
	}

	public void setDeliveryDate(Date deliveryDate) {
		this.deliveryDate = deliveryDate;
	}

	public Restaurant getRestaurant() {
		return restaurant;
	}

	protected void setRestaurant(Restaurant restaurant) {
		this.restaurant = (Restaurant) restaurant;
		// biz logic that should be elsewhere
		setLineItems(new PendingOrderLineItemCollection());
	}

	class PendingOrderLineItemComparator implements Comparator {

		public int compare(Object arg0, Object arg1) {
			int x = ((PendingOrderLineItem) arg0).getIndex();
			int y = ((PendingOrderLineItem) arg1).getIndex();
			if (x < y)
				return -1;
			if (x == y)
				return 0;
			return 1;
		}

	}

	public PendingOrderLineItemCollection getLineItems() {
		TreeSet set = new TreeSet(new PendingOrderLineItemComparator());
        set.addAll(lineItems);
		ArrayList result = new ArrayList();
		result.addAll(set);
		return new PendingOrderLineItemCollection(result);
	}

	protected void setLineItems(PendingOrderLineItemCollection lineItems) {
		this.lineItems = (ArrayList) lineItems.getItems();
	}

	public PaymentInformation getPaymentInformation() {
		if (paymentName == null)
			return null;
        Address paymentAddress = new Address(paymentStreet1, paymentStreet2, paymentCity, paymentState, paymentZip);
		return new PaymentInformation(
			paymentType,
			paymentName,
			paymentNumber,
			paymentMonth,
			paymentYear,
			paymentAddress);
	}

	protected void setPaymentInformation(PaymentInformation paymentInformation) {
		if (paymentInformation == null) {
            paymentType = null;
            paymentName = null;
            paymentNumber = null;
            paymentMonth = 0;
            paymentYear = 0;

            paymentStreet1 = null;
            paymentStreet2 = null;
            paymentCity = null;
            paymentState = null;
            paymentZip = null;
        } else {
            paymentType = paymentInformation.getType();
            paymentName = paymentInformation.getName();
            paymentNumber = paymentInformation.getNumber();
            paymentMonth = paymentInformation.getMonth();
            paymentYear = paymentInformation.getYear();

            paymentStreet1 = paymentInformation.getAddress().getStreet1();
            paymentStreet2 = paymentInformation.getAddress().getStreet2();
            paymentCity = paymentInformation.getAddress().getCity();
            paymentState = paymentInformation.getAddress().getState();
            paymentZip = paymentInformation.getAddress().getZip();
        }
	}

	protected void setDeliveryAddress(Address address) {
		if (address == null) {
			this.deliveryStreet1 = null;
			this.deliveryStreet2 = null;
			this.deliveryCity = null;
			this.deliveryState = null;
			this.deliveryZip = null;
		} else {
			this.deliveryStreet1 = address.getStreet1();
			this.deliveryStreet2 = address.getStreet2();
			this.deliveryCity = address.getCity();
			this.deliveryState = address.getState();
			this.deliveryZip = address.getZip();
		}
	}

	public Address getDeliveryAddress() {
		if (deliveryZip == null)
			return null;
		else
			return new Address(
				deliveryStreet1,
				deliveryStreet2,
				deliveryCity,
				deliveryState,
				deliveryZip);
	}

	protected PendingOrderLineItem makeLineItem(int index, MenuItem miv) {
		return new PendingOrderLineItem(this, index, miv);
	}

	protected void setAuthorizationTransaction(AuthorizationTransaction txn) {
		// TODO - fix me
	}

}
