package net.chrisrichardson.foodToGo.order;

import java.util.*;

import junit.framework.*;

import net.chrisrichardson.foodToGo.order.testImpl.*;
import net.chrisrichardson.foodToGo.util.*;

/*
 * Implements the TestCases for a PendingOrder
 * It is implemented this way so that we can use the same tests for the entity bean
 * implementation - ie. Cactus, which requires use to extend a particular test case
 * base class
 */
 
public class PendingOrderTestHelper {

    PendingOrder pendingOrder;
    Restaurant restaurant1;
    Restaurant restaurant1a;
    Restaurant restaurant2;

    private Date goodDeliveryTime = PendingOrderTestData.makeGoodDeliveryTime();

	public PendingOrderTestHelper(
		PendingOrder pendingOrder,
		PendingOrderTestData testData) {
		this.pendingOrder = pendingOrder;
		this.restaurant1 = testData.restaurant1;
		this.restaurant1a = testData.restaurant1a;
		this.restaurant2 = testData.restaurant2;
	}

    void assertDeliveryAddress(Address expected) throws Exception {
        Assert.assertEquals("Address", expected, pendingOrder.getDeliveryAddress());
    }

    public void assertRestaurant(Restaurant expectedRestaurant) throws Exception {
        Assert.assertEquals(
            "restaurant",
            expectedRestaurant,
            pendingOrder.getRestaurant());
    }

    private void assertDeliveryTime(Date expectedDeliveryTime) {
        Assert.assertEquals(
            "deliveryTime",
            expectedDeliveryTime,
            pendingOrder.getDeliveryDate());
    }

    private void assertState(int state) {
        Assert.assertEquals("state", state, pendingOrder.getState());
    }

    ///

    private void assertState_PLACED() {
        assertState(PendingOrder.PLACED);
    }

    private void assertState_READY_TO_PLACE() {
        assertState(PendingOrder.READY_TO_PLACE);
    }

    private void assertState_READY_FOR_CHECKOUT() {
        assertState(PendingOrder.READY_FOR_CHECKOUT);
        Assert.assertEquals(
            "subtotal",
            124.5,
            pendingOrder.getSubtotal(),
            0.1);
    }

    private void assertState_RESTAURANT_SELECTED(Restaurant expectedRestaurant)
        throws Exception {
        assertState(PendingOrder.RESTAURANT_SELECTED);
        assertRestaurant(expectedRestaurant);
        Assert.assertTrue("lineItems", !pendingOrder.getLineItems().isEmpty());
    }

    private void assertState_DELIVERY_ADDRESS_AND_TIME_SPECIFIED(Address expectedAddress)
        throws Exception {
        assertState(PendingOrder.DELIVERY_ADDRESS_AND_TIME_SPECIFIED);
        assertDeliveryAddress(expectedAddress);
        assertDeliveryTime(goodDeliveryTime);
        assertRestaurant(null);
    }

    private void assertState_NEW() throws Exception {
        Assert.assertEquals("state", PendingOrder.NEW, pendingOrder.getState());
        assertDeliveryAddress(null);
        assertDeliveryTime(null);
    }

    private void gotoRestaurantSelected() throws Exception {
        assertState_NEW();

        int result =
            pendingOrder.updateDeliveryAddressAndDate(
                PendingOrderTestData.ADDRESS1,
                goodDeliveryTime,
                false);

        Assert.assertEquals(PendingOrder.OK, result);
        assertState_DELIVERY_ADDRESS_AND_TIME_SPECIFIED(PendingOrderTestData.ADDRESS1);

        boolean success = pendingOrder.updateRestaurant(restaurant1);

        checkpoint();

        Assert.assertTrue("success", success);
        assertState_RESTAURANT_SELECTED(restaurant1);

    }

    protected void checkpoint() {
    }

    //

    public void test_normal() throws Exception {
        gotoReadyToPlace();
        String result = pendingOrder.placeOrder();
        checkpoint();

        assertState_PLACED();
        Assert.assertNotNull("order id", result);
    }

    public void gotoReadyToPlace() throws Exception, InvalidPendingOrderStateException {
        gotoReadyForCheckout();
        pendingOrder.updatePaymentInformation(PendingOrderTestData.PAYMENT_INFORMATION);
        checkpoint();

        assertState_READY_TO_PLACE();
    }

    public void gotoReadyForCheckout()
        throws Exception, InvalidPendingOrderStateException {
 
        gotoRestaurantSelected();
 
        MenuItemCollection menuItems = restaurant1.findMenuItems();
        int[] testQuantities = new int[menuItems.size()];
        for (int index = 0; index < menuItems.size(); index++) {
            testQuantities[index] = 10;
		}
        pendingOrder.updateQuantities(testQuantities);
        
        checkpoint();

        assertState_READY_FOR_CHECKOUT();

        boolean success = pendingOrder.checkout();
        Assert.assertTrue("checkout success", success);
        assertState_READY_FOR_CHECKOUT();

    }

    public void testChangeDeliveryAddress_compatibleWithSelectedRestaurant()
        throws Exception {
        gotoRestaurantSelected();
        int result =
            pendingOrder.updateDeliveryAddressAndDate(
                PendingOrderTestData.ADDRESS1A,
                goodDeliveryTime,
                false);
        Assert.assertEquals(PendingOrder.OK, result);

        assertState_RESTAURANT_SELECTED(restaurant1);
        assertDeliveryAddress(PendingOrderTestData.ADDRESS1A);
    }

    public void testChangeDeliveryAddress_incompatibleWithSelectedRestaurant()
        throws Exception {
        gotoRestaurantSelected();
        int result =
            pendingOrder.updateDeliveryAddressAndDate(
                PendingOrderTestData.ADDRESS2,
                goodDeliveryTime,
                true);
        Assert.assertEquals(PendingOrder.OK, result);

        assertState_DELIVERY_ADDRESS_AND_TIME_SPECIFIED(PendingOrderTestData.ADDRESS2);
    }

    public void testUpdateDeliveryAddressAndDate_bad() throws Exception {
        int result =
            pendingOrder.updateDeliveryAddressAndDate(
                PendingOrderTestData.BAD_ADDRESS,
                goodDeliveryTime,
                false);
        Assert.assertEquals(PendingOrder.INVALID_DELIVERY_ADDRESS_OR_TIME, result);
        assertState_NEW();
    }

    public void testUpdateRestaurant_incompatibleWithAddress() throws Exception {
        Assert.assertEquals(
            PendingOrder.OK,
            pendingOrder.updateDeliveryAddressAndDate(
                PendingOrderTestData.ADDRESS1,
                goodDeliveryTime,
                false));
        boolean success = pendingOrder.updateRestaurant(restaurant2);
        Assert.assertTrue("success", !success);

        assertState_DELIVERY_ADDRESS_AND_TIME_SPECIFIED(PendingOrderTestData.ADDRESS1);
    }

    public void testUpdateRestaurant_different() throws Exception {
        gotoRestaurantSelected();
        boolean success = pendingOrder.updateRestaurant(restaurant1a);
        Assert.assertTrue("success", success);

        assertState_RESTAURANT_SELECTED(restaurant1a);
    }

    /*
        public void testCheckout() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            boolean booleanRet = pendingOrder.checkout();
    
        }
    
        public void testGetDeliveryAddress() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Address addressRet = pendingOrder.getDeliveryAddress();
    
        }
    
        public void testGetDeliveryCharges() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            double doubleRet = pendingOrder.getDeliveryCharges();
    
        }
    
        public void testGetDeliveryDate() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Date dateRet = pendingOrder.getDeliveryDate();
    
        }
    
        public void testGetLineItems() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            PendingOrderLineItemCollection pendingorderlineitemcollectionRet = pendingOrder.getLineItems();
    
        }
    
        public void testGetPaymentInformation() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            PaymentInformation paymentinformationRet = pendingOrder.getPaymentInformation();
    
        }
    
        public void testGetRestaurant() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            IRestaurant irestaurantRet = pendingOrder.getRestaurant();
    
        }
    
        public void testGetSalesTax() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            double doubleRet = pendingOrder.getSalesTax();
    
        }
    
        public void testGetState() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            int intRet = pendingOrder.getState();
    
        }
    
        public void testGetSubtotal() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            double doubleRet = pendingOrder.getSubtotal();
    
        }
    
        public void testGetTotal() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            double doubleRet = pendingOrder.getTotal();
    
        }
    
        public void testIsCompatibleWithSelectedRestaurant() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Address deliveryAddress1 = null  ;
            Date deliveryTime2 = null  ;
            boolean booleanRet = pendingOrder.isCompatibleWithSelectedRestaurant(deliveryAddress1, deliveryTime2);
    
        }
    
        public void testMeetsMinimumOrder() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            boolean booleanRet = pendingOrder.meetsMinimumOrder();
    
        }
    
        public void testPlaceOrder() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            int longRet = pendingOrder.placeOrder();
    
        }
    
        public void testSetDeliveryAddress() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Address deliveryAddress1 = null  ;
            pendingOrder.setDeliveryAddress(deliveryAddress1);
    
        }
    
        public void testSetDeliveryDate() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Date deliveryDate1 = null  ;
            pendingOrder.setDeliveryDate(deliveryDate1);
    
        }
    
        public void testSetLineItems() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            PendingOrderLineItemCollection lineItems1 = null  ;
            pendingOrder.setLineItems(lineItems1);
    
        }
    
        public void testSetMenuItems() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            MenuItemCollection menuItems1 = null  ;
            pendingOrder.setMenuItems(menuItems1);
    
        }
    
        public void testSetPaymentInformation() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            PaymentInformation paymentInformation1 = null  ;
            pendingOrder.setPaymentInformation(paymentInformation1);
    
        }
    
        public void testSetRestaurant() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            IRestaurant restaurant1 = null  ;
            pendingOrder.setRestaurant(restaurant1);
    
        }
    
        public void testUpdateLineItemQuantities() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Map lineItems1 = null  ;
            pendingOrder.updateLineItemQuantities(lineItems1);
    
        }
    
        public void testUpdatePaymentInformation() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            PaymentInformation paymentInformation1 = null  ;
            pendingOrder.updatePaymentInformation(paymentInformation1);
    
        }
    
        public void testUpdateQuantities() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Map lineItems1 = null  ;
            pendingOrder.updateQuantities(lineItems1);
    
        }
    
    
        public void testVerifyDeliveryAddress() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Address deliveryAddress1 = null  ;
            Address addressRet = pendingOrder.verifyDeliveryAddress(deliveryAddress1);
    
        }
    
        public void testVerifyDeliveryTime() throws Exception {
            trm = new TestRestaurantManager();
            pendingOrder = new PendingOrder(trm);
            Date deliveryTime1 = null  ;
            Date dateRet = pendingOrder.verifyDeliveryTime(deliveryTime1);
    
        }
     */
}