package net.chrisrichardson.foodToGo.orderService.impl;

import java.rmi.*;
import java.util.*;

import junit.framework.*;

import net.chrisrichardson.foodToGo.creditCardProcessing.*;
import net.chrisrichardson.foodToGo.order.*;
import net.chrisrichardson.foodToGo.orderService.values.*;
import net.chrisrichardson.foodToGo.util.*;
 
 
 /*
  * Base class for DBProcessOrderTests
  * Extended by test cases for the DBProcessOrderServiceBean 
  * and the DBProcessOrderManager
  * 
  * The tests are written as Template Methods.
  * Subclasses must override configure() to setup the environment correctly
  * Subclasses must also define the methods that invoke methods on the target class
  * The SessionBean or the POJO manager
  */
  
public abstract class BaseDBProcessOrderTests extends TestCase {

    protected Date goodDeliveryTime = PendingOrderTestData.makeGoodDeliveryTime();
    protected String pendingOrderId;

	private String restaurantId1;
    private String restaurantId2;

    public BaseDBProcessOrderTests(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        super.setUp();
        pendingOrderId = null;
        configure();
        restaurantId1 = PendingOrderTestData.RESTAURANT_ID1;
        restaurantId2 = PendingOrderTestData.RESTAURANT_ID2;
    }

    /**
     * Defined by subclasses to configure the environment correctly
     */
    
	protected abstract void configure();


    private void gotoRestaurantSelected() throws RemoteException {
        gotoDeliveryAddressAndTimeEnter();
        SelectRestaurantResponse result2 =
            selectRestaurant(pendingOrderId, restaurantId1);
        pendingOrderId = result2.getPendingOrderId();

        assertEquals(SelectRestaurantResponse.OK, result2.getStatus());
        assertPendingOrderId();
    }


    private void gotoDeliveryAddressAndTimeEnter() throws RemoteException {
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.ADDRESS1,
                goodDeliveryTime);
        pendingOrderId = result.getPendingOrderId();

        assertPendingOrderId();
    }

    private void gotoCheckedOut() throws RemoteException {
        gotoReadyToCheckout();
        CheckoutResponse result = checkout(pendingOrderId);
        pendingOrderId = result.getPendingOrderId();

        assertEquals(CheckoutResponse.OK, result.getStatus());
        assertPendingOrderId();
    }

    private void gotoReadyToPlace() throws RemoteException {
        gotoCheckedOut();
        EnterPaymentInformationResponse result =
            enterPaymentInformation(pendingOrderId, PendingOrderTestData.PAYMENT_INFORMATION);
        pendingOrderId = result.getPendingOrderId();

        assertEquals(EnterPaymentInformationResponse.OK, result.getStatus());
        assertPendingOrderId();
    }

    private void gotoReadyToCheckout() throws RemoteException {
        gotoRestaurantSelected();
        UpdateQuantitiesResponse result =
            updateQuantities(pendingOrderId, PendingOrderTestData.getTEST_QUANTITIES());
        pendingOrderId = result.getPendingOrderId();

        assertEquals(UpdateQuantitiesResponse.OK, result.getStatus());
        assertPendingOrderId();
    }

    public void assertPendingOrderId(ProcessOrderServiceResponse result2) {
        assertNotNull("pendingOrderId", result2.getPendingOrderId());
    }

    public void assertPendingOrderId() {
        assertNotNull("pendingOrderId", pendingOrderId);
    }

    // End Of Make methods

    public void testEnterDeliveryAddressAndTime_newOrder() throws Exception {
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.ADDRESS1,
                goodDeliveryTime);
        pendingOrderId = result.getPendingOrderId();
        assertPendingOrderId();
        assertTrue(!result.getRestaurants().isEmpty());
    }

    public void testEnterDeliveryAddressAndTime_newOrder_badAddress() throws Exception {
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.BAD_ADDRESS,
                goodDeliveryTime);
        assertEquals(
            EnterDeliveryAddressAndTimeResponse.NO_RESTAURANT_AVAILABLE,
            result.getStatus());
        assertPendingOrderId(result);
    }

    // coverage showed this was missing.

    public void testEnterDeliveryAddressAndTime_incompatibleWithSelectedRestaurantAndInvalid()
        throws Exception {
        gotoRestaurantSelected();
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.BAD_ADDRESS,
                goodDeliveryTime);
        assertEquals(
            EnterDeliveryAddressAndTimeResponse.NO_RESTAURANT_AVAILABLE,
            result.getStatus());
        assertPendingOrderId(result);
    }

    public void testEnterDeliveryAddressAndTime_incompatibleWithSelectedRestaurant()
        throws Exception {
        gotoRestaurantSelected();
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.ADDRESS2,
                goodDeliveryTime);
        pendingOrderId = result.getPendingOrderId();

        assertEquals(
            EnterDeliveryAddressAndTimeResponse.CONFIRM_CHANGE,
            result.getStatus());
        assertPendingOrderId();
    }

    public void testEnterDeliveryAddressAndTime_compatibleWithSelectedRestaurant()
        throws Exception {
        gotoRestaurantSelected();
        EnterDeliveryAddressAndTimeResponse result =
            enterDeliveryAddressAndTime(
                pendingOrderId,
                PendingOrderTestData.ADDRESS1A,
                goodDeliveryTime);
        assertEquals(
            EnterDeliveryAddressAndTimeResponse.KEEP_RESTAURANT,
            result.getStatus());
        assertPendingOrderId(result);
    }

    public void testConfirmDeliveryAddressAndTimeChange() throws Exception {
        testEnterDeliveryAddressAndTime_incompatibleWithSelectedRestaurant();
        EnterDeliveryAddressAndTimeResponse result =
            confirmDeliveryAddressAndTimeChange(
                pendingOrderId,
                PendingOrderTestData.ADDRESS2,
                goodDeliveryTime);
        assertEquals(
            EnterDeliveryAddressAndTimeResponse.SELECT_RESTAURANT,
            result.getStatus());
        assertPendingOrderId(result);
    }

    public void testSelectRestaurant_compatible() throws Exception {
        gotoRestaurantSelected();
    }

    public void testSelectRestaurant_incompatible() throws Exception {
        gotoDeliveryAddressAndTimeEnter();
        SelectRestaurantResponse result2 =
            selectRestaurant(pendingOrderId, restaurantId2);
        assertEquals(SelectRestaurantResponse.SELECT_RESTAURANT, result2.getStatus());
        assertPendingOrderId(result2);
    }

    public void testUpdateQuantities() throws Exception {
        gotoReadyToCheckout();
    }

    public void testUpdateQuantities_zero() throws Exception {
        gotoRestaurantSelected();
        UpdateQuantitiesResponse result =
            updateQuantities(pendingOrderId, PendingOrderTestData.getZERO_QUANTITIES());
        pendingOrderId = result.getPendingOrderId();
        assertEquals(UpdateQuantitiesResponse.OK, result.getStatus());
        assertPendingOrderId(result);
    }

    public void testCheckout() throws Exception {
        gotoCheckedOut();
    }

    public void testEnterPaymentInformation() throws Exception {
        gotoReadyToPlace();
    }

    public void testPlace() throws Exception {
        gotoReadyToPlace();
        PlaceOrderResponse result = placeOrder(pendingOrderId);
        assertEquals(PlaceOrderResponse.OK, result.getStatus());
        assertPendingOrderId(result);
    }

    // This are the abstract methods for invoking methods on the object being tested
    
    protected abstract PlaceOrderResponse placeOrder(String pendingOrderId)  throws RemoteException;
    protected abstract EnterDeliveryAddressAndTimeResponse enterDeliveryAddressAndTime(
        String pendingOrderId,
        Address address,
        Date goodDeliveryTime)  throws RemoteException;

    protected abstract SelectRestaurantResponse selectRestaurant(
        String pendingOrderId,
        String restaurantId)  throws RemoteException;
    protected abstract CheckoutResponse checkout(String pendingOrderId)  throws RemoteException;
    protected abstract EnterPaymentInformationResponse enterPaymentInformation(
        String pendingOrderId,
        PaymentInformation paymentInformation)  throws RemoteException;

    protected abstract UpdateQuantitiesResponse updateQuantities(
        String pendingOrderId,
        int[] quantities)  throws RemoteException;

    protected abstract EnterDeliveryAddressAndTimeResponse confirmDeliveryAddressAndTimeChange(
        String pendingOrderId,
        Address address,
        Date goodDeliveryTime)  throws RemoteException;

}
