package net.chrisrichardson.foodToGo.orderService.impl;

/* Copyright (c) 2002 Chris Richardson. All Rights Reserved */

import java.rmi.*;
import java.util.*;

import javax.ejb.*;

import net.chrisrichardson.foodToGo.creditCardProcessing.*;
import net.chrisrichardson.foodToGo.order.*;
import net.chrisrichardson.foodToGo.orderService.*;
import net.chrisrichardson.foodToGo.orderService.values.*;
import net.chrisrichardson.foodToGo.util.*;

/*
 * Implements business methods of the session facade
 * Each of the business methods corresponds to the steps of the usecase
 */
 
public class DBProcessOrderManager extends BasicManager {


	PendingOrder findOrCreatePendingOrder(String pendingOrderId) {
		if (pendingOrderId == null)
			return PendingOrderRepository.createPendingOrder();
		else
			return PendingOrderRepository.findPendingOrder(pendingOrderId);
	}

	public EnterDeliveryAddressAndTimeResponse enterDeliveryAddressAndTime(
		String pendingOrderId,
		Address deliveryAddress,
		Date deliveryTime) {

		open();
		try {

			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);

			int result =
				pendingOrder.updateDeliveryAddressAndDate(
					deliveryAddress,
					deliveryTime,
					false);

			PendingOrderValue pov =
				PendingOrderValueFactory.makePendingOrderValue(pendingOrder);

			switch (result) {
				case PendingOrder.OK :
					if (pendingOrder.getState()
						== PendingOrder.DELIVERY_ADDRESS_AND_TIME_SPECIFIED) {
						return new EnterDeliveryAddressAndTimeResponse(
							EnterDeliveryAddressAndTimeResponse.SELECT_RESTAURANT,
							PendingOrderRepository.getObjectId(pendingOrder),
							pov,
							getRestaurants(
								pendingOrder.getDeliveryAddress(),
								pendingOrder.getDeliveryDate()));
					} else {
						return new EnterDeliveryAddressAndTimeResponse(
							EnterDeliveryAddressAndTimeResponse.KEEP_RESTAURANT,
							PendingOrderRepository.getObjectId(pendingOrder),
							pov,
							null);
					}
				case PendingOrder.INVALID_DELIVERY_ADDRESS_OR_TIME :
					return new EnterDeliveryAddressAndTimeResponse(
						EnterDeliveryAddressAndTimeResponse.NO_RESTAURANT_AVAILABLE,
						PendingOrderRepository.getObjectId(pendingOrder),
						pov,
						null);
				case PendingOrder.CONFIRM_CHANGE :
					return new EnterDeliveryAddressAndTimeResponse(
						EnterDeliveryAddressAndTimeResponse.CONFIRM_CHANGE,
						PendingOrderRepository.getObjectId(pendingOrder),
						pov,
						null);
				default :
					throw new NotYetImplementedException("result = " + result);
			}

		} finally {
			close();
		}
	}

	RestaurantValuePagedCollection getRestaurants(
		Address deliveryAddress,
		Date deliveryTime) {
		return RestaurantRepository.getRestaurantsThatServeZipCodeAndTime(
			deliveryAddress,
			deliveryTime);
	}

	public EnterDeliveryAddressAndTimeResponse confirmDeliveryAddressAndTimeChange(
		String pendingOrderId,
		Address pendingDeliveryAddress,
		Date pendingDeliveryDate) {

		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);

			if (pendingDeliveryAddress == null || pendingDeliveryDate == null) {
				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
				return new EnterDeliveryAddressAndTimeResponse(
					EnterDeliveryAddressAndTimeResponse.BAD_STATE,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov,
					null);
			}

			int result =
				pendingOrder.updateDeliveryAddressAndDate(
					pendingDeliveryAddress,
					pendingDeliveryDate,
					true);

			PendingOrderValue pov =
				PendingOrderValueFactory.makePendingOrderValue(pendingOrder);

			switch (result) {
				case PendingOrder.OK :
					if (pendingOrder.getState()
						== PendingOrder.DELIVERY_ADDRESS_AND_TIME_SPECIFIED) {
						return new EnterDeliveryAddressAndTimeResponse(
							EnterDeliveryAddressAndTimeResponse.SELECT_RESTAURANT,
							PendingOrderRepository.getObjectId(pendingOrder),
							pov,
							getRestaurants(
								pendingOrder.getDeliveryAddress(),
								pendingOrder.getDeliveryDate()));
					} else {
						return new EnterDeliveryAddressAndTimeResponse(
							EnterDeliveryAddressAndTimeResponse.KEEP_RESTAURANT,
							PendingOrderRepository.getObjectId(pendingOrder),
							pov,
							null);
					}
				case PendingOrder.INVALID_DELIVERY_ADDRESS_OR_TIME :
					return new EnterDeliveryAddressAndTimeResponse(
						EnterDeliveryAddressAndTimeResponse.NO_RESTAURANT_AVAILABLE,
						PendingOrderRepository.getObjectId(pendingOrder),
						pov,
						null);
				case PendingOrder.CONFIRM_CHANGE :
				default :
					throw new NotYetImplementedException("result = " + result);
			}
		} finally {
			close();
		}
	}

	public SelectRestaurantResponse selectRestaurant(
		String pendingOrderId,
		String restaurantId) {

		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);

			Restaurant restaurant = RestaurantRepository.findRestaurant(restaurantId);

			boolean success;
			try {
				success = pendingOrder.updateRestaurant(restaurant);

				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);

				return new SelectRestaurantResponse(
					success
						? SelectRestaurantResponse.OK
						: SelectRestaurantResponse.SELECT_RESTAURANT,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov);

			} catch (InvalidPendingOrderStateException e) {
				throw new EJBException(e);
			}

		} finally {
			close();
		}
	}

	public UpdateQuantitiesResponse updateQuantities(
		String pendingOrderId,
		int[] quantities) {

		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);

			try {
				pendingOrder.updateQuantities(quantities);
				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
				return new UpdateQuantitiesResponse(
					UpdateQuantitiesResponse.OK,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov);
			} catch (InvalidPendingOrderStateException e) {
				throw new EJBException(e);
			}
		} finally {
			close();
		}
	}

	public CheckoutResponse checkout(String pendingOrderId) {

		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);
			try {
				boolean success = pendingOrder.checkout();
				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
				return new CheckoutResponse(
					success ? CheckoutResponse.OK : CheckoutResponse.DOES_NOT_MEET_MINIMUM,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov);
			} catch (InvalidPendingOrderStateException e) {
				throw new EJBException(e);
			}
		} finally {
			close();
		}
	}

	public EnterPaymentInformationResponse enterPaymentInformation(
		String pendingOrderId,
		PaymentInformation paymentInformation) {
		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);
			try {
				pendingOrder.updatePaymentInformation(paymentInformation);
				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
				return new EnterPaymentInformationResponse(
					EnterPaymentInformationResponse.OK,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov);
			} catch (InvalidPendingOrderStateException e) {
				throw new EJBException(e);
			}
		} finally {
			close();
		}
	}

	public PlaceOrderResponse placeOrder(String pendingOrderId) {

		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);
			try {
				String orderNumber = pendingOrder.placeOrder();
				PendingOrderValue pov =
					PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
				return new PlaceOrderResponse(
					PlaceOrderResponse.OK,
					PendingOrderRepository.getObjectId(pendingOrder),
					pov,
					orderNumber);
			} catch (InvalidPendingOrderStateException e) {
				throw new EJBException(e);
			}

		} finally {
			close();
		}
	}

	public PendingOrderValue getPendingOrder(String pendingOrderId) {
		open();
		try {
			PendingOrder pendingOrder = findOrCreatePendingOrder(pendingOrderId);
			return PendingOrderValueFactory.makePendingOrderValue(pendingOrder);
		} finally {
			close();
		}
	}

}
